package com.diagbot.util;

import cn.hutool.core.codec.Base64;
import com.diagbot.dto.ImageCaptchaDTO;
import com.diagbot.entity.ImageCaptchaParams;
import org.patchca.color.SingleColorFactory;
import org.patchca.filter.predefined.CurvesRippleFilterFactory;
import org.patchca.font.RandomFontFactory;
import org.patchca.service.ConfigurableCaptchaService;
import org.patchca.utils.encoder.EncoderHelper;
import org.patchca.word.RandomWordFactory;

import java.awt.*;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * @author huangrusheng
 * @version 1.0
 * @date 2021/5/20 9:54
 */
public final class ImageCaptchaUtil {

    private ImageCaptchaUtil(){}
    /**
     * 流方式输出
     * @param params
     * @param outputStream
     * @return 图片文字内容
     * @throws IOException
     */
    public static String create(ImageCaptchaParams params, OutputStream outputStream) throws IOException{
        ConfigurableCaptchaService  captchaService = new ConfigurableCaptchaService();
        captchaService.setColorFactory(new SingleColorFactory(Color.black));
        CurvesRippleFilterFactory curvesRippleFilterFactory = new CurvesRippleFilterFactory();
        curvesRippleFilterFactory.setColorFactory(captchaService.getColorFactory());
        curvesRippleFilterFactory.setStrokeMin(1);
        curvesRippleFilterFactory.setStrokeMax(1.5f);
        captchaService.setFilterFactory(curvesRippleFilterFactory);
        captchaService.setHeight(params.getHeight());
        captchaService.setWidth(params.getWidth());
        RandomWordFactory wordFactory = new RandomWordFactory();
        wordFactory.setCharacters(params.getWords());
        wordFactory.setMaxLength(params.getMaxWordAmount());
        wordFactory.setMinLength(params.getMinWordAmount());
        captchaService.setWordFactory(wordFactory);
        RandomFontFactory fontFactory = new RandomFontFactory();
        fontFactory.setMaxSize(params.getMaxFontSize());
        fontFactory.setMinSize(params.getMinFontSize());
        captchaService.setFontFactory(fontFactory);
        return EncoderHelper.getChallangeAndWriteImage(captchaService,params.getFormat(),outputStream);
    }
    /**
     * 以base64方式返回
     * @param params
     * @return 图片文字内容+base64格式的图片
     * @throws IOException
     */
    public static ImageCaptchaDTO createBase64(ImageCaptchaParams params) throws IOException{
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream(1024 * 3);
        String words = create(params,outputStream);
        return new ImageCaptchaDTO(words, Base64.encode(outputStream.toByteArray()),"");
    }
    /**
     * 以Web的img识别的base64返回
     * @param params
     * @return 图片文字内容+base64格式的img标签识别的图片
     * @throws IOException
     */
    public static ImageCaptchaDTO createBase64ForWeb(ImageCaptchaParams params) throws IOException{
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream(1024 * 3);
        String words = create(params,outputStream);
        String encode = Base64.encode(outputStream.toByteArray());
        return new ImageCaptchaDTO(words,encode,"");
    }
}
